/*
 * GaussianNoise.h
 *
 *  Created on: 2022/06/02
 *      Author: T.Anazawa
 *      Outline: Add gaussian noise for wheels.
 */

#ifndef CNOID_TURTLEBOT_CONTROL_INCLUDE_NOISE_GENERATOR_H
#define CNOID_TURTLEBOT_CONTROL_INCLUDE_NOISE_GENERATOR_H
#include <cnoid/Referenced>
#include <random>
#include <ros/node_handle.h>

class NoiseGenerator : public cnoid::Referenced
{
	struct NoiseParam {
		double detect_rate;
		double mean;
		double stddev;
	};

	NoiseParam* np;

public:
	// Constructor.
	NoiseGenerator();
	// Destructor.
	~NoiseGenerator();
	void initializeGaussianNoise();
	bool detectionValue();
//	double addGaussianNoise();
	std::pair<double, double> addGaussianNoise();

	// Getter.
	double detectionRate() const { return np->detect_rate; };
	double mean() const { return np->mean; };
	double standardDeviation() const { return np->stddev; };

	// Setter.
	void setDetectionRate(double detect_rate) { np->detect_rate = detect_rate; };
	void setMean(double mean) { np->mean = mean; };
	void setStandardDeviation(double stddev) { np->stddev = stddev; };

private:
	ros::NodeHandle nh_;
	std::mt19937 random_num;
	std::uniform_real_distribution<> detection_probability;
	std::normal_distribution<> rotation_error_distribution;
};

typedef cnoid::ref_ptr<NoiseGenerator> NoiseGeneratorPtr;

#endif /* CNOID_TURTLEBOT_CONTROL_INCLUDE_NOISE_GENERATOR_H */
